import React, { Component } from 'react';
import { render } from 'react-dom';
import { Provider, connect } from 'react-redux';
import { createStore, combineReducers } from 'redux';

// Action Creators creates Actions
function addTodo(text) {
  return { type: 'ADD_TODO', text };
}

function completeTodo(index) {
  return { type: 'COMPLETE_TODO', index };
}

function setVisibilityFilter(filter) {
  return { type: 'SET_VISIBILITY_FILTER', filter };
}

// Reducer (s)
function visibilityFilter(state = 'SHOW_ALL', action) {
  switch (action.type) {
  case 'SET_VISIBILITY_FILTER':
    return action.filter;
  default:
    return state;
  }
}

function todos(state = [], action) {
  switch (action.type) {
    case 'ADD_TODO':
      return [
        ...state,
        {
          text: action.text,
          completed: false
        }
      ];
    case 'COMPLETE_TODO':
      return [
        ...state.slice(0, action.index),
        Object.assign({}, state[action.index], { completed: true }),
        ...state.slice(action.index + 1)
      ];
    default:
      return state;
  }
}

const todoApp = combineReducers({ visibilityFilter, todos });

const initialState = {
  visibilityFilter: 'SHOW_ALL',
  todos: [
    {
      text: 'Consider using Rexux',
      completed: true
    },
    {
      text: 'keep all state in a single tree',
      completed: false
    }
  ]
};

let store = createStore(todoApp, initialState);

store.subscribe(() => console.log(store.getState()));

store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_ALL' });
store.dispatch({ type: 'ADD_TODO',  text: 'Web+DB writing' });
// store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_COMPLETED' });
store.dispatch(setVisibilityFilter('SHOW_COMPLETED'));
store.dispatch({ type: 'COMPLETE_TODO',  index: 1 });

//
// Connect with React
//
const App = () => (
  <div>
    <h1>Hello, Redux</h1>
    <VisibleTodoList />
    <FilterFooter />
  </div>
);

const TodoList = ({todos}) => {
  const items = todos.map((todo) => {
    return <li
            style={{textDecoration: todo.completed? 'line-through': 'none' }}>{todo.text}</li>;
  });
  return <ul>{items}</ul>;
};

const Footer = ({filter, onFilterChange}) => {
  const renderFilter = (currentFilter, name) => {
    if (currentFilter === filter) {
      return name;
    }

    return (
      <a href='#' onClick={e => {
        e.preventDefault();
        onFilterChange(currentFilter);
      }}>
        {name}
      </a>
    );
  };

  return (
    <p>
      Show:
      {' '}
      {renderFilter('SHOW_ALL', 'All')}
      {', '}
      {renderFilter('SHOW_COMPLETED', 'Completed')}
      {', '}
      {renderFilter('SHOW_ACTIVE', 'Active')}
      .
    </p>
  );
};

// Container Components
//
// VisibleTodoList:
//
function getVisibleTodos(todos, filter) {
  switch(filter) {
    case 'SHOW_ALL':
      return todos;
    case 'SHOW_COMPLETED':
      return todos.filter(todo => todo.completed);
    case 'SHOW_ACTIVE':
      return todos.filter(todo => !todo.completed);
  }
}

const mapTodoStateToProps = (state) => {
  return { todos: getVisibleTodos(state.todos, state.visibilityFilter) };
};

// const mapDispatchToProps = (dispatch) => {
//   return { onTodoClick: (id) => { dispatch(toggleTodo(id)); } };
// };

const VisibleTodoList = connect(mapTodoStateToProps)(TodoList);

//
// FilterFooter:
//
const mapFooterStateToProps = (state) => {
  return { filter: state.visibilityFilter };
};

const mapFooterDispatchToProps = (dispatch) => {
  return {
    onFilterChange: (filter) => { dispatch(setVisibilityFilter(filter)); }
  };
};

const FilterFooter = connect(mapFooterStateToProps, mapFooterDispatchToProps)(Footer);

render(
  <Provider store={store}>
    <App />
  </Provider>,
  document.getElementById('container')
);
